Index: xbmc/Settings.h
===================================================================
--- xbmc/Settings.h	(revision 17537)
+++ xbmc/Settings.h	(working copy)
@@ -231,6 +231,7 @@
 #ifdef HAS_SDL
     bool m_fullScreen;
     bool m_startFullScreen;
+    bool m_fitToScreen;
 #endif
     int m_playlistRetries;
     int m_playlistTimeout;
Index: xbmc/XBVideoConfig.cpp
===================================================================
--- xbmc/XBVideoConfig.cpp	(revision 17537)
+++ xbmc/XBVideoConfig.cpp	(working copy)
@@ -126,10 +126,13 @@
   int screen = DefaultScreen(pRootDisplay);
   int width = DisplayWidth(pRootDisplay, screen);
   int height = DisplayHeight(pRootDisplay, screen);
-  XineramaScreenInfo *info;
+  XineramaScreenInfo *info = NULL;
   int num;
-  info = XineramaQueryScreens(pRootDisplay, &num);
-  if (info)
+  // XineramaQueryScreens returns bogus information when using separate screen dual heading
+  // with Nvidia drivers. This may happen with other drivers also. Specifically it returns
+  // the resolution of the first screen even if run on the second screen. In any case if
+  // Xinerama is not active, it should not be trusted.
+  if (XineramaIsActive(pRootDisplay) && (info = XineramaQueryScreens(pRootDisplay, &num)))
   {
     int desired = 0;
     width = info[0].width;
@@ -352,7 +355,7 @@
 #ifdef _WIN32PC
   // Some drivers won't display HDTC_720p or other resolutions correct.
   // We'll return DESKTOP when started in fullscreen to be safe.
-  if(g_advancedSettings.m_startFullScreen == true)
+  if(g_advancedSettings.m_startFullScreen == true || g_advancedSettings.m_fitToScreen == true)
     return DESKTOP;
 #endif
   // Get the desktop resolution to see what we're dealing with here.
Index: xbmc/Settings.cpp
===================================================================
--- xbmc/Settings.cpp	(revision 17537)
+++ xbmc/Settings.cpp	(working copy)
@@ -262,6 +262,7 @@
 
 #ifdef HAS_SDL
   g_advancedSettings.m_fullScreen = false;
+  g_advancedSettings.m_fitToScreen = false;
 #endif
 
   g_advancedSettings.m_playlistRetries = 100;
@@ -1260,6 +1261,7 @@
   XMLUtils::GetBoolean(pRootElement, "disablemodchipdetection", g_advancedSettings.m_DisableModChipDetection);
 #ifdef HAS_SDL
   XMLUtils::GetBoolean(pRootElement, "fullscreen", g_advancedSettings.m_startFullScreen);
+  XMLUtils::GetBoolean(pRootElement, "fittoscreen", g_advancedSettings.m_fitToScreen);
 #endif
 
   GetInteger(pRootElement, "songinfoduration", g_advancedSettings.m_songInfoDuration, 1, 15);
Index: guilib/GraphicContext.cpp
===================================================================
--- guilib/GraphicContext.cpp	(revision 17537)
+++ guilib/GraphicContext.cpp	(working copy)
@@ -606,6 +606,16 @@
     g_xrandr.RestoreState();
 #endif
   }
+
+  // Dirty hack to add "fit to screen" video mode.
+  // Overrides all other video settings.
+  if (g_advancedSettings.m_fitToScreen)
+  {
+    g_advancedSettings.m_fullScreen = false;
+    m_bFullScreenRoot = false;
+    res = DESKTOP;
+    CLog::Log(LOGDEBUG, "Advanced setting fittoscreen set, forcing desktop resolution.");
+  }
   
   if (res==WINDOW)
   {
@@ -622,11 +632,17 @@
 
 #ifdef HAS_SDL_2D
     int options = SDL_HWSURFACE | SDL_DOUBLEBUF;
-    if (g_advancedSettings.m_fullScreen) options |= SDL_FULLSCREEN;
+    if (g_advancedSettings.m_fitToScreen)
+        options |= SDL_NOFRAME;
+    else if (g_advancedSettings.m_fullScreen)
+        options |= SDL_FULLSCREEN;
     m_screenSurface = new CSurface(m_iScreenWidth, m_iScreenHeight, true, 0, 0, 0, (bool)g_advancedSettings.m_fullScreen);
 #elif defined(HAS_SDL_OPENGL)
     int options = SDL_RESIZABLE;
-    if (g_advancedSettings.m_fullScreen) options |= SDL_FULLSCREEN;
+    if (g_advancedSettings.m_fitToScreen)
+        options |= SDL_NOFRAME;
+    else if (g_advancedSettings.m_fullScreen)
+        options |= SDL_FULLSCREEN;
 
     // Create a bare root window so that SDL Input handling works
 #ifdef HAS_GLX
